#include <Wire.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>

#define I2C_SLAVEADDR_BQ2589x  (0x6a) // I2Cスレーブアドレス
#define POLL_PERIOD (2000) // 測定周期 = 2sec
#define NOTIFY_PERIOD (1000*60) // 通知周期 = 1min

const char* ssid = "Wi-FiアクセスポイントのSSID";
const char* password = "WiFiアクセスポイントに接続するパスワード";

#define IFTTT_KEY     "IFTTT Maker Channelのキーを設定する"

#define IFTTT_HOST    "maker.ifttt.com"
#define IFTTT_EVENT_STATUS "BatteryStatus"
#define IFTTT_EVENT_BATTV "BatteryVoltage"

uint8_t regread(uint8_t addr);
void regwrite(uint8_t addr, uint8_t val);

/*
  チャージャーICのADC変換開始を指示する
*/
void set_oneshot_measurement(void)
{
  uint8_t val;

  val = regread(0x02); // REG02の値を読み出し
  val &= ~(1 << 6); // One shot ADC conversionを設定
  val |= 0x80; // Start ADC Conversionを設定
  regwrite(0x02, val); // チャージャーICに値を書き込み
}

void setup(void) {
  uint8_t val;

  Serial.begin(115200);

  Wire.begin(4,5); // I2C初期化

  WiFi.begin(ssid, password);
  Serial.println("");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // チャージャ設定
  val = regread(0x00);
  regwrite(0x00, val & (~(1 << 6)));

  val = regread(0x02);
  regwrite(0x02, val & (~(1 << 4)));

  val = regread(0x07);
  regwrite(0x07, val & (~(3 << 4)));
}

void send_ifttt(String event_name, String v1, String v2, String v3)
{
  WiFiClient client;
  String path;
  String json;
  path = "/trigger/" + event_name + "/with/key/" + String(IFTTT_KEY);

  json = "{ \"value1\" : \"" + v1 + "\", \"value2\" : \"" + v2 + "\", \"value3\" : \"" + v3 + "\" }";

  if (client.connect(IFTTT_HOST, 80)) {
    client.print(String("POST ") + path + " HTTP/1.1\r\n" +
                 "Host: " + String(IFTTT_HOST) + "\r\n" +
                 "Content-Length: " + String(strlen(json.c_str())) + "\r\n" +
                 "Content-Type: application/json\r\n\r\n" +
                 json + "\r\n" /*
                 "Connection: close\r\n\r\n"*/);

    Serial.println("Notified to IFTTT Maker channel");
    Serial.println(json);
    Serial.println(String("POST ") + path + " HTTP/1.1\r\n" +
                   "Host: " + String(IFTTT_HOST) + "\r\n" +
                   "Content-Type: application/json\r\n" +
                   "Content-Length: " + String(strlen(json.c_str())) + "\r\n" +
                   json + "\r\n" /*
                 "Connection: close\r\n\r\n"*/);

  } else {
    Serial.println("connection failed");
  }
}

void loop(void) {
  static uint32_t next_measure_time = 0;
  static uint32_t next_notify_time = 0;
  static int batt_level = 0;
  static bool first_time = true;
  int i;
  uint8_t val;
  float batt_voltage;
  bool charged;

  if (next_measure_time < millis()) {
    val = regread(0x0b); // REG0Bの読み出し
    charged = (((val & 0x18) >> 3) == 0x03) ? true : false; // チャージ完了であればtrueが代入される
    val = regread(0x0e);
    batt_voltage = (2.304f + (((float)(val & 0x7f)) * 0.02f));
    Serial.print("REG0B="); Serial.println(val, HEX);
    Serial.print("BattV="); Serial.println(batt_voltage);

    if (first_time == true) {
      send_ifttt(IFTTT_EVENT_STATUS, "---", String(batt_voltage), "");
      first_time = false;
    }

    if (next_notify_time < millis()) {
      send_ifttt(IFTTT_EVENT_BATTV, String(batt_voltage), "", "");
      Serial.println("BattV notified");
      next_notify_time = millis() + NOTIFY_PERIOD;
    }

    if ((batt_level == 0) && (batt_voltage >= 3.96f)) {
      batt_level = 1;
      send_ifttt(IFTTT_EVENT_STATUS, "30%", String(batt_voltage), "");
    } else if ((batt_level == 1) && (batt_voltage >= 4.10f)) {
      batt_level = 2;
      send_ifttt(IFTTT_EVENT_STATUS, "70%", String(batt_voltage), "");
    } else if ((batt_level == 2) && (charged == true)) {
      batt_level = 3;
      send_ifttt(IFTTT_EVENT_STATUS, "Fully", String(batt_voltage), "");
    }
    set_oneshot_measurement();
    next_measure_time = millis() + POLL_PERIOD;
  }
}

// デバイスからのレジスタ値の読み出し処理
uint8_t regread(uint8_t addr) {
  int err;
  uint8_t val;

  Wire.beginTransmission(I2C_SLAVEADDR_BQ2589x);
  Wire.write(addr);
  Wire.endTransmission(false);
  Wire.requestFrom(I2C_SLAVEADDR_BQ2589x, 1, false);
  val = Wire.read();
  Wire.endTransmission();

  return val; // リードした値を返す
}

// デバイスのレジスタへの書き込み処理
void regwrite(uint8_t addr, uint8_t val) {
  int err;

  Wire.beginTransmission(I2C_SLAVEADDR_BQ2589x);
  Wire.write(addr);
  Wire.write(val);
  Wire.endTransmission(false);

}
